SystemOrganization addCategory: #'ToolsTest-Inspector'!
SystemOrganization addCategory: #'ToolsTest-MessageTally'!
SystemOrganization addCategory: #'ToolsTest-PointerFinder'!

TestCase subclass: #MessageTallyTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolsTest-MessageTally'!

!MessageTallyTest methodsFor: 'tallying' stamp: 'Alexandre Bergel 3/5/2010 17:53'!
testClosedTally

	| tally |
	tally := MessageTally
				tallySendsTo: nil 
				inBlock:  [ 3.14159 printString ] 
				showTree: true 
				closeAfter: true 
				openResultWindow: false.
				
	"The tally is closed, therefore nothing may be accessed"
	self assert: (tally receivers isNil)! !

!MessageTallyTest methodsFor: 'basic' stamp: 'Alexandre Bergel 3/4/2010 18:44'!
testInstantiation

	| messageTally |
	messageTally := MessageTally new.
	self assert: (messageTally time = 0).
	self assert: (messageTally tally = 0)! !

!MessageTallyTest methodsFor: 'sampling' stamp: 'Alexandre Bergel 3/15/2010 21:03'!
testSampling1

	| tally |
	tally := MessageTally
				spyOn: [ 1000  timesRepeat: [ 3.14159 printString  ] ]  
				reportOtherProcesses: false 
				cutoff: 1 
				openResultWindow: false
				closeAfter: false.

	"No process is accessible from the tally, the reason is that using the sampling method execute the block in the current process"
	self assert: (tally process isNil).

! !

!MessageTallyTest methodsFor: 'tallying' stamp: 'md 6/24/2010 16:26'!
testTallySends

	| tally thisMethod activeProcess allProcesses numberOfProcesses processUsedByTally tallyForPrintString tallyForPrintStringBase tallyForSmallIntegerStar |
	allProcesses := Process allInstances.
	numberOfProcesses := allProcesses size.

	tally := MessageTally
				tallySendsTo: nil 
				inBlock:  [ 3.14159 printString ] 
				showTree: true 
				closeAfter: false 
				openResultWindow: false.
	
	"tally is for this test method"
	thisMethod := self class >> #testTallySends.
	self assert: (tally hash = thisMethod hash).
	self assert: (tally method == thisMethod).
	self assert: (tally theClass == self class).


	processUsedByTally := tally process.
	self deny: (allProcesses includes: processUsedByTally).

	"Below is specialized for the execution ' 3.14159 printString '. Any method change in the execution flow of printString will turn this test to yellow"
	self assert: (tally tally = 78).
	"The tally is not close, therefore the tree may be crawled over"
	self assert: (tally receivers isArray).
	self assert: (tally receivers size = 2).
	
	"--------"
	tallyForPrintString := tally receivers second.
	"Since 3.14159 is a float"
	self assert: (tallyForPrintString theClass == Float).
	"the executed method is Number>>printString"
	self assert: (tallyForPrintString method == ( Number>>#printString)).
	self assert: (tallyForPrintString tally = 75).

	"--------"
	tallyForPrintStringBase := tallyForPrintString receivers first.
	"The receiver is still a Float"
	self assert: (tallyForPrintString theClass == Float).
	"the executed method is Number>>printStringBase: this time"
	self assert: (tallyForPrintStringBase method == ( Number>>#printStringBase:)).
	self assert: (tallyForPrintStringBase tally = 74).
	"The method printStringBase: calls two methods, SequenceableCollection class >> streamContents: and Float >> printOn:base:"
	self assert: (tallyForPrintStringBase receivers size = 2).
	
	"streamContents: is been tallied 13 times and printOn:base: 59 times"
	self assert: (tallyForPrintStringBase receivers collect: #tally) asArray = #(13 59).	

	"--------"
	tallyForSmallIntegerStar := tallyForPrintStringBase receivers second receivers second receivers third.
	self assert: (tallyForSmallIntegerStar theClass == SmallInteger).

	"We close to explicitely release reference of the process, the class and methods"
	tally close.
	
	
! !

!MessageTallyTest methodsFor: 'tallying' stamp: 'Alexandre Bergel 3/5/2010 17:29'!
testTallyShouldNotContainsAReturn

	"The block passed to MessageTally should not have a ^ "
	
	self should: [ MessageTally
					tallySendsTo: nil 
					inBlock:  [ ^ 3.14159 printString ] 
					showTree: true 
					closeAfter: false 
					openResultWindow: false ] raise: Error.
	! !

TestCase subclass: #PointerFinderTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolsTest-PointerFinder'!

!PointerFinderTest methodsFor: 'as yet unclassified' stamp: 'Alexandre.Bergel 7/4/2009 11:14'!
testBasic1

	| myObject myArray |
	myObject := Object new.
	myArray := {myObject . myObject}.
	self assert: (PointerFinder pointersTo: myObject) asArray = {myArray}! !

!PointerFinderTest methodsFor: 'as yet unclassified' stamp: 'Alexandre.Bergel 7/4/2009 11:24'!
testCycle

	| myObject myArray myArray2 pointingObjects |
	myObject := Object new.
	myArray := {myObject . myObject}.
	myArray2 := {myObject . myArray}.

	pointingObjects := (PointerFinder pointersTo: myObject) asArray.
	self assert: pointingObjects size = 2.
	self assert: (pointingObjects includesAllOf: {myArray . myArray2}).
	
	"PointerFinder loops in presence of cycles"
"	myArray at: 1 put: myArray.
	pointingObjects := (PointerFinder pointersTo: myObject) asArray.
	self assert: pointingObjects = {myArray}.
"! !

!PointerFinderTest methodsFor: 'as yet unclassified' stamp: 'Alexandre.Bergel 7/4/2009 11:13'!
testNoPointingObject

	| myObject |
	myObject := Object new.
	self assert: (PointerFinder pointersTo: myObject) isEmpty! !

TestCase subclass: #WeakSetInspectorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolsTest-Inspector'!
